package com.ltt.compass.weather

import android.content.Context
import android.graphics.Color
import android.os.Bundle
import android.util.Log
import android.view.View
import android.widget.ImageView
import android.widget.RelativeLayout
import android.widget.TextView
import androidx.core.widget.NestedScrollView
import androidx.swiperefreshlayout.widget.SwipeRefreshLayout
import api.ttfeed.Express_API_TT
import com.dotools.umlibrary.UMPostUtils
import com.dtbus.ggs.KGSManager
import com.google.android.material.snackbar.Snackbar
import com.ltt.compass.R
import com.ltt.compass.mvp.blankj.BusProvider
import com.ltt.compass.utils.GlobalConfig
import com.ltt.compass.weather.base.BaseMvpFragment
import com.ltt.compass.weather.bean.*
import com.ltt.compass.weather.contract.WeatherContract
import com.ltt.compass.weather.event.CloseEvent
import com.ltt.compass.weather.event.OnReciveEvent
import com.ltt.compass.weather.listener.IWeatherFragmentCallBack
import com.ltt.compass.weather.presenter.WeatherPresenterImp
import com.ltt.compass.weather.util.DialogUtil
import com.ltt.compass.weather.util.DisplayUtils
import com.ltt.compass.weather.util.LocationManager
import com.ltt.compass.weather.util.WeatherUtils
import com.ltt.compass.weather.widget.WeatherDetailView
import com.ltt.compass.weather.widget.WeatherHeadView
import com.ltt.compass.weather.widget.WeatherIndicesView
import com.ltt.compass.weather.widget.daily.DailyDayWeatherView
import com.ltt.compass.weather.widget.hour.HourView
import com.ltt.compass.weather.widget.hour.IndexHorizontalScrollView


class WeatherFragment : BaseMvpFragment<WeatherPresenterImp>(), WeatherContract.IWeatherView {
    private var weatherCallBack: IWeatherFragmentCallBack? = null
    private var cityId = ""

    companion object {
        fun create(cityId: String): WeatherFragment {
            val fragment = WeatherFragment()
            val bundle = Bundle()
            bundle.putString(AppConfig.CITY_ID, cityId)
            fragment.arguments = bundle
            return fragment
        }
    }

    /**
     * 头部
     */
    private lateinit var mHeadView: WeatherHeadView//头部父布局
    private lateinit var mScrollView: NestedScrollView//滑动布局

    /**
     * 详情
     */
    private lateinit var mDetailView: WeatherDetailView//头部父布局

    //24小时
    private lateinit var mHourScrollView: IndexHorizontalScrollView
    private lateinit var mHourView: HourView

    //十天
    private lateinit var mDailyView: DailyDayWeatherView

    //指数
    private lateinit var mIndicesView: WeatherIndicesView

    //刷新
    private lateinit var mSwipeRefreshLayout: SwipeRefreshLayout

    //当天天气数据
    private var mCurrentWeatherData: CurrentWeatherData.CurrentWeatherDataBean? = null

    //头条信息流1
    private var mTTViewOne: View? = null

    //头条信息流2
    private var mTTViewTwo: View? = null

    private var mScrollY = 0

    var title: TextView? = null
    var weatherTxt: ImageView? = null
    var back: ImageView? = null

    override fun onAttach(context: Context) {
        super.onAttach(context)
        lifecycle.addObserver(LocationManager(context))
        BusProvider.getBus().register(this)

        BusProvider.getBus().subscribe<OnReciveEvent>(this) { onReciveEvent -> //TODO 事件处理
            this.cityId = onReciveEvent.cityId;
            title!!.text = onReciveEvent.location
            updateWeather(false)
            mSwipeRefreshLayout.isRefreshing = false
        }
    }


    override fun getContentViewId(): Int {
        return R.layout.weather_fragment
    }

    override fun initViews() {
        mSwipeRefreshLayout = mContentView!!.findViewById(R.id.weather_refresh)
        mSwipeRefreshLayout.setProgressBackgroundColorSchemeResource(android.R.color.white)
        mSwipeRefreshLayout.setColorSchemeColors(Color.BLUE, Color.YELLOW, Color.GREEN)
        mScrollView = mContentView!!.findViewById(R.id.nested_scrollView)
        title = mContentView!!.findViewById(R.id.weather_title)
        weatherTxt = mContentView!!.findViewById(R.id.weather_click)
        back = mContentView!!.findViewById(R.id.back)
        mHeadView = mContentView!!.findViewById(R.id.weather_head_view)
        mHeadView.initViews()
        mDetailView = mContentView!!.findViewById(R.id.weather_details_view)
        mDetailView.initViews()
        //信息流
        mTTViewOne = mContentView!!.findViewById(R.id.tt_item_one)
        mTTViewTwo = mContentView!!.findViewById(R.id.tt_item_two)
        //24
        mHourScrollView = mContentView!!.findViewById(R.id.hour_weather_scroll)
        mHourView = mContentView!!.findViewById(R.id.hour_view)
        //10
        mDailyView = mContentView!!.findViewById(R.id.daily_weather_view)
        //指数
        mIndicesView = mContentView!!.findViewById(R.id.weather_indices_view)
        mIndicesView.initViews()
        weatherTxt?.setOnClickListener {
            DialogUtil.showDownload(context)
        }
        back?.setOnClickListener {
            activity!!.finish()
        }
        if (GlobalConfig.getOverSevenTask(activity)) {
            mTTViewOne!!.visibility = View.GONE
            mTTViewTwo!!.visibility = View.GONE
        }
    }

    override fun initData() {
        mSwipeRefreshLayout.setOnRefreshListener {
            mSwipeRefreshLayout.isRefreshing = false
        }
        this.cityId = GlobalConfig.CITYID
        mPresenter.initDb()
        updateWeather(false)
    }


    override fun initPresenter(): WeatherPresenterImp {
        return WeatherPresenterImp()
    }

    override fun updateWeather(needRefresh: Boolean) {
        mSwipeRefreshLayout.isRefreshing = needRefresh
        mPresenter.getCurrentWeather(cityId, needRefresh)
        mPresenter.getHourWeather(cityId, needRefresh)
        mPresenter.getDailyDayWeather(cityId, needRefresh)
        mPresenter.getObservations(cityId, needRefresh)
        mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.CAPSULE)
    }


    override fun getCurrentWeatherSuccess(currentWeather: CurrentWeatherData.CurrentWeatherDataBean) {
        this.mCurrentWeatherData = currentWeather
        //回调MainActivity设置天气动画
        weatherCallBack?.weatherViewTypeChange(cityId, currentWeather.WeatherIcon, currentWeather.IsDayTime)
        //设置当前天气温度 icon 和描述 当前天气没有今天的最高和最低温度 得去10日预报中设置今天最高和最低
        mHeadView.setCurrentDegrees(currentWeather.Temperature!!)
        mHeadView.setWeatherText(currentWeather.WeatherText!!)
        //设置天气详情的风，湿度，体感温度
        mDetailView.setWind(currentWeather.Wind!!)
        mDetailView.setHumidity(currentWeather.RelativeHumidity.toString())
        mDetailView.setSensation(currentWeather.ApparentTemperature!!)

        if (KGSManager.getKGStatus(KGSManager.GDT, mContext.applicationContext)) {
            if (!GlobalConfig.getOverSevenTask(activity)) {
                setTT(mTTViewOne!!, AppConfig.TT_INFO_ID_ONE, 0)
                setTT(mTTViewTwo!!, AppConfig.TT_INFO_ID_TWO, 1)
            }

        }
        if (KGSManager.getKGStatus(KGSManager.GDT, mContext.applicationContext)) {
            weatherTxt!!.visibility = View.VISIBLE
        } else {
            weatherTxt!!.visibility = View.GONE
        }
        mSwipeRefreshLayout.isRefreshing = false
    }

    override fun getHourWeatherSuccess(hourWeather: List<HourWeatherData.HourWeatherDataBean>) {
        //设置24小时天气
        mHourScrollView.setToday24HourView(mHourView)
        mHourView.setData(hourWeather)
    }

    override fun getObservationSuccess(observationData: ObservationData.ObservationDataBean) {
        //设置当前的空气质量
        mHeadView.setObservationData(observationData)
    }


    override fun getIndicesDailySuccess(indicesDaily: List<WeatherIndices.WeatherIndicesBean>, indicesType: WeatherUtils.WeatherIndicesType) {
//        TODO("Not yet implemented")
        mIndicesView.setText(indicesDaily[0], indicesType)
        when (indicesType) {
            WeatherUtils.WeatherIndicesType.CAPSULE -> {
                mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.FLIGHT)
            }
            WeatherUtils.WeatherIndicesType.FLIGHT -> {
                mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.FISHING)
            }
            WeatherUtils.WeatherIndicesType.FISHING -> {
                mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.TELESCOPE)
            }
            WeatherUtils.WeatherIndicesType.TELESCOPE -> {
                mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.SPORTS)
            }
            WeatherUtils.WeatherIndicesType.SPORTS -> {
                mPresenter.getIndicesDaily(cityId, WeatherUtils.WeatherIndicesType.TRIP)
            }
        }
    }

    override fun getDailyDayWeatherSuccess(dailyWeather: DailyDayWeatherData.DailyDayWeatherDataBean) {
        //设置今天的最高和最低天气温度
        mHeadView.setToDayDegrees(dailyWeather.DailyForecasts!![0].Temperature!!.Maximum!!, dailyWeather.DailyForecasts!![0].Temperature!!.Minimum!!)
        //设置日出日落和降雨概率
        mDetailView.setSunTime(dailyWeather.DailyForecasts!![0].Sun!!)
        if (mCurrentWeatherData != null) {
            if (mCurrentWeatherData!!.IsDayTime) {
                mDetailView.setRainNum(dailyWeather.DailyForecasts!![0].Day!!.PrecipitationProbability.toString())
            } else {
                mDetailView.setRainNum(dailyWeather.DailyForecasts!![0].Night!!.PrecipitationProbability.toString())
            }
        }
        //设置10日天气
        //画曲线
        mDailyView.lineType = DailyDayWeatherView.LINE_TYPE_CURVE;
        //设置线宽
        mDailyView.lineWidth = 3f
        //设置白天和晚上线条的颜色
        mDailyView.setDayAndNightLineColor(Color.parseColor("#FFA300"), Color.parseColor("#95C7DF"))
        //点击某一列
        mDailyView.setOnWeatherItemClickListener { itemView, position -> }
        mDailyView.setData(dailyWeather.DailyForecasts)
    }

    override fun showError(error: String) {
        mSwipeRefreshLayout.isRefreshing = false
        Snackbar.make(mHeadView, resources.getText(R.string.weather_error), Snackbar.LENGTH_SHORT).show()

    }

    override fun showMessage(message: String) {
        Log.e("joker", "showMessage: " + message)
    }


    //设置头条广告
    private val tt = Express_API_TT.getInstance()

    private fun setTT(view: View, id: String, index: Int) {

        val item = view.findViewById<RelativeLayout>(R.id.tt_item)!!
        if (tt != null) {

            tt.LoadTTExpress(context, AppConfig.TTAPPID, id, 300, 0, 1, false, item, object : Express_API_TT.TTExpressListener {
                override fun onLoad(size: Int) {
                    view.visibility = View.VISIBLE
                    Log.e("TTERROR", "onLoad")
                    if (index == 0) {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO1_PULLSUCESS)
                    } else {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO2_PULLSUCESS)
                    }
                }

                override fun onObClicked(type: Int) {
                    if (index == 0) {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO1_CLICK)
                    } else {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO2_CLICK)
                    }
                    Log.e("TTERROR", "onObClicked")

                }

                override fun onObShow(type: Int) {
                    if (index == 0) {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO1_SHOW)
                    } else {
                        UMPostUtils.onEvent(context, AppConfig.LIST_VIDEO2_SHOW)
                    }
                    Log.e("TTERROR", "onObShow")

                }

                override fun onRenderFail(msg: String?, code: Int) {
                    view.visibility = View.GONE
                    Log.e("TTERROR", "onRenderFail")

                }

                override fun onRenderSuccess() {
                    Log.e("TTERROR", "onRenderSuccess")

                }


                override fun onError(code: Int, message: String?) {
                    val map = HashMap<String, String>()
                    map["error"] = " code:$code msg:$message"
                    if (index == 0) {
                        UMPostUtils.onEventMap(context, AppConfig.LIST_VIDEO1_FAILED, map)
                    } else {
                        UMPostUtils.onEventMap(context, AppConfig.LIST_VIDEO2_FAILED, map)
                    }
                    view.visibility = View.GONE
                    Log.e("TTERROR", "$code---$message")
                }
            })
        } else {
            view.visibility = View.GONE
            Log.e("TTERROR", "tt  null")

        }
    }

    override fun onResume() {
        super.onResume()
        //回调改变现实的动画view和标题栏
        if (mCurrentWeatherData != null) {
            weatherCallBack?.weatherViewTypeChange(cityId, mCurrentWeatherData!!.WeatherIcon, mCurrentWeatherData!!.IsDayTime)
            weatherCallBack?.setToolbarColor(DisplayUtils.getColorForScrollY(mHeadView.height, mScrollY))
        }
    }

    override fun onDestroy() {
        super.onDestroy()
//        broadcastManager?.unregisterReceiver(mReceiver!!)
        BusProvider.getBus().unregister(this)

    }
}
