package com.ltt.compass.weather.base

import android.content.Context
import android.content.Intent
import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.fragment.app.Fragment
import androidx.fragment.app.FragmentActivity
import com.tools.permissions.library.Permissions

abstract class BaseMvpFragment<P : BasePresenter<*>> : Fragment(), IBaseView, BaseViewInit {
    protected lateinit var mPresenter: P
    protected val pers = arrayOf(Permissions.READ_EXTERNAL_STORAGE, Permissions.WRITE_EXTERNAL_STORAGE, Permissions.READ_PHONE_STATE)
    protected var mContentView: View? = null
    protected lateinit var mContext: Context

    override fun onAttach(context: Context) {
        super.onAttach(context)
        mContext = context
    }

    override fun onCreateView(
            inflater: LayoutInflater,
            container: ViewGroup?,
            savedInstanceState: Bundle?
    ): View? {
        mContentView = inflater.inflate(getContentViewId(), container, false)
        initViews()
        return mContentView
    }

    override fun onActivityCreated(savedInstanceState: Bundle?) {
        super.onActivityCreated(savedInstanceState)
        mPresenter = initPresenter()
        mPresenter.attachView(this)
        initData()
    }

    /**初始化presenter**/
    abstract fun initPresenter(): P

    protected fun startActivity(activity: FragmentActivity) {
        val intent = Intent(mContext, activity::class.java)
        mContext.startActivity(intent)
    }

    protected fun startActivity(activity: FragmentActivity, bundle: Bundle) {
        val intent = Intent(mContext, activity::class.java)
        intent.putExtras(bundle)
        mContext.startActivity(intent)
    }


    protected fun startActivityResult(activity: FragmentActivity, bundle: Bundle) {
        val intent = Intent(mContext, activity::class.java)
        intent.putExtras(bundle)
        mContext.startActivity(intent)
    }

    override fun getContext(): Context {
        return mContext
    }


}