package com.ltt.compass.weather.model

import android.content.ContentValues
import android.content.Context
import com.google.gson.Gson
import com.ltt.compass.weather.bean.CurrentWeatherData
import com.ltt.compass.weather.bean.DailyDayWeatherData
import com.ltt.compass.weather.bean.HourWeatherData
import com.ltt.compass.weather.bean.ObservationData
import com.ltt.compass.weather.contract.WeatherContract
import com.ltt.compass.weather.util.DataBaseOpenHelper


class WeatherModelImp : WeatherContract.IWeatherModel {
    private var mHelper: DataBaseOpenHelper? = null
    private var mGson = Gson()

    override fun initDb(context: Context) {
        if (mHelper == null) {
            mHelper = DataBaseOpenHelper.getInstance(context, DataBaseOpenHelper.DB_NAME, DataBaseOpenHelper.DB_VERSION)
        }
    }

    override fun closeDb() {
        mHelper?.close()
    }


    override fun getWeather(cityId: String): CurrentWeatherData? {
        val cursor = mHelper!!.query(DataBaseOpenHelper.DB_TABLE_NAME, "where " + DataBaseOpenHelper.CITY_ID + "=$cityId")
        return if (cursor.count > 0) {
            var weatherData: CurrentWeatherData? = null
            while (cursor.moveToNext()) {
                val weatherJson = cursor.getString(cursor.getColumnIndex(DataBaseOpenHelper.CURRENT_WEATHER_JSON))
                if (weatherJson != null) {
                    weatherData = mGson.fromJson(weatherJson, CurrentWeatherData::class.java)
                }
                closeDb()
            }
            weatherData
        } else {
            closeDb()
            null
        }
    }

    override fun saveWeather(cityId: String, weatherData: CurrentWeatherData) {
        val value = ContentValues()
        value.put(DataBaseOpenHelper.CURRENT_WEATHER_JSON, mGson.toJson(weatherData))
        mHelper!!.update(DataBaseOpenHelper.DB_TABLE_NAME, value, DataBaseOpenHelper.CITY_ID + "= ?", arrayOf(cityId))
        closeDb()
    }

    override fun getHourWeather(cityId: String): HourWeatherData? {
        val cursor = mHelper!!.query(DataBaseOpenHelper.DB_TABLE_NAME, "where " + DataBaseOpenHelper.CITY_ID + "=$cityId")
        return if (cursor.count > 0) {
            var hourData:HourWeatherData? =null
            while (cursor.moveToNext()) {
                val weatherJson = cursor.getString(cursor.getColumnIndex(DataBaseOpenHelper.HOUR_WEATHER_JSON))
                if (weatherJson != null) {
                    hourData = mGson.fromJson(weatherJson,HourWeatherData::class.java)
                }
                closeDb()
            }
            hourData
        } else {
            closeDb()
            null
        }
    }

    override fun saveHourWeather(cityId: String, hourWeatherData: HourWeatherData) {
        val value = ContentValues()
        value.put(DataBaseOpenHelper.HOUR_WEATHER_JSON, mGson.toJson(hourWeatherData))
        mHelper!!.update(DataBaseOpenHelper.DB_TABLE_NAME, value, DataBaseOpenHelper.CITY_ID + "= ?", arrayOf(cityId))
        closeDb()
    }

    override fun getDailyDayWeather(cityId: String): DailyDayWeatherData? {
        val cursor = mHelper!!.query(DataBaseOpenHelper.DB_TABLE_NAME, "where " + DataBaseOpenHelper.CITY_ID + "=$cityId")
        return if (cursor.count > 0) {
            var dailyData :DailyDayWeatherData? = null
            while (cursor.moveToNext()) {
                val weatherJson = cursor.getString(cursor.getColumnIndex(DataBaseOpenHelper.DAILY_WEATHER_JSON))
                if (weatherJson != null) {
                    dailyData = mGson.fromJson(weatherJson, DailyDayWeatherData::class.java)
                }
                closeDb()
            }
            dailyData
        } else {
            closeDb()
            null
        }
    }

    override fun saveDailyDayWeather(cityId: String, dailyWeatherData: DailyDayWeatherData) {
        val value = ContentValues()
        value.put(DataBaseOpenHelper.DAILY_WEATHER_JSON, mGson.toJson(dailyWeatherData))
        mHelper!!.update(DataBaseOpenHelper.DB_TABLE_NAME, value, DataBaseOpenHelper.CITY_ID + "= ?", arrayOf(cityId))
        closeDb()
    }

    override fun getObservation(cityId: String): ObservationData? {
        val cursor = mHelper!!.query(DataBaseOpenHelper.DB_TABLE_NAME, "where " + DataBaseOpenHelper.CITY_ID + "=$cityId")
        return if (cursor.count > 0) {
            var observationData: ObservationData? = null
            while (cursor.moveToNext()) {
                val weatherJson = cursor.getString(cursor.getColumnIndex(DataBaseOpenHelper.OBSERVATIONS_JSON))
                if (weatherJson != null) {
                    observationData = mGson.fromJson(weatherJson, ObservationData::class.java)
                }
                closeDb()
            }
            observationData
        } else {
            closeDb()
            null
        }
    }

    override fun saveObservation(cityId: String, observationData: ObservationData) {
        val value = ContentValues()
        value.put(DataBaseOpenHelper.OBSERVATIONS_JSON, mGson.toJson(observationData))
        mHelper!!.update(DataBaseOpenHelper.DB_TABLE_NAME, value, DataBaseOpenHelper.CITY_ID + "= ?", arrayOf(cityId))
        closeDb()
    }
}