package com.ltt.compass.weather.util


import android.app.AlertDialog
import android.content.Context
import android.content.Intent
import android.net.Uri
import android.view.Gravity
import android.view.LayoutInflater
import android.view.ViewGroup
import android.widget.Button
import android.widget.ImageView
import android.widget.TextView
import android.widget.Toast
import com.dotools.umlibrary.UMPostUtils
import com.ltt.compass.R
import com.ltt.compass.weather.bean.ObservationData
import com.ltt.compass.weather.widget.CircleProgress
import java.lang.ref.WeakReference

object DialogUtil {
    private var mDialog: AlertDialog? = null
    private var mContext: WeakReference<Context>? = null

    interface LocationBtnClickListener {
        fun onOpenClick()
        fun onAddClick()
    }

    interface VideoBtnClickListener {
        fun onOkClick()
        fun onNoClick()
    }


    private fun initDialog() {
        // 设置参数
        val params = mDialog!!.window.attributes
        // 宽度和高度
        params.width = ViewGroup.LayoutParams.WRAP_CONTENT
        params.height = ViewGroup.LayoutParams.WRAP_CONTENT
        params.gravity = Gravity.CENTER
        params.windowAnimations = R.style.DialogAnimStyle
        mDialog!!.window.attributes = params
        mDialog!!.show()
    }

    //    fun showLoadingDialog(context: Context) {
//        mContext = WeakReference(context)
//        val builder = AlertDialog.Builder(mContext!!.get(), R.style.DialogStyle)
//        val view = LayoutInflater.from(mContext!!.get()).inflate(R.layout.dialog_load_layout, null, false)
//        mDialog = builder.setView(view).create()
//        mDialog!!.setCanceledOnTouchOutside(false)//点击屏幕不消失
//        initDialog()
//    }
//
//
//    fun showLocationErrorDialog(context: Context, isNoPermission: Boolean, clickListener: LocationBtnClickListener) {
//        mContext = WeakReference(context)
//        val builder = AlertDialog.Builder(mContext!!.get(), R.style.DialogStyle)
//        val view = LayoutInflater.from(mContext!!.get()).inflate(R.layout.dialog_location_layout, null, false)
//        val title = view.findViewById<TextView>(R.id.location_dialog_title)
//        val msg = view.findViewById<TextView>(R.id.location_dialog_msg)
//        val openBtn = view.findViewById<Button>(R.id.dialog_open)
//        val addText = view.findViewById<TextView>(R.id.dialog_add)
//        if (isNoPermission) {
//            msg.visibility = View.VISIBLE
//            title.text = mContext!!.get()!!.resources.getString(R.string.open_location_service)
//            openBtn.text = mContext!!.get()!!.resources.getString(R.string.open_location)
//        } else {
//            msg.visibility = View.GONE
//            title.text = mContext!!.get()!!.resources.getString(R.string.location_error)
//            openBtn.text = mContext!!.get()!!.resources.getString(R.string.reset_location)
//        }
//        openBtn.setOnClickListener {
//            dialogDismiss()
//            clickListener.onOpenClick()
//        }
//        addText.setOnClickListener {
//            dialogDismiss()
//            clickListener.onAddClick()
//        }
//        mDialog = builder.setView(view).create()
//        initDialog()
//    }
//
//
//
//    fun showVideoDialog(context: Context,  clickListener: VideoBtnClickListener) {
//        mContext = WeakReference(context)
//        val builder = AlertDialog.Builder(mContext!!.get(), R.style.DialogStyle)
//        val view = LayoutInflater.from(mContext!!.get()).inflate(R.layout.dialog_video_layout, null, false)
//        val okBtn = view.findViewById<Button>(R.id.dialog_ok)
//        val noText = view.findViewById<TextView>(R.id.dialog_cancel)
//        okBtn.setOnClickListener {
//            dialogDismiss()
//            clickListener.onOkClick()
//        }
//        noText.setOnClickListener {
//            dialogDismiss()
//            clickListener.onNoClick()
//        }
//        mDialog = builder.setView(view).create()
//        initDialog()
//    }
//
    fun showDownload(context: Context) {
        UMPostUtils.onEvent(context,"more_city_click")
        mContext = WeakReference(context)
        val builder = AlertDialog.Builder(mContext!!.get(), R.style.DialogStyle)
        val view = LayoutInflater.from(mContext!!.get()).inflate(R.layout.weather_dialog, null, false)
        val close = view.findViewById<Button>(R.id.weather_cancel)
        val download = view.findViewById<Button>(R.id.weather_download)
        close.setOnClickListener { dialogDismiss() }
        download.setOnClickListener { dialogDismiss()
            UMPostUtils.onEvent(context,"download_weather_click")
            try {
                val uri = Uri.parse("market://details?id=com.dotools.weather")
                val goToMarket = Intent(Intent.ACTION_VIEW, uri)
                context.startActivity(goToMarket)
            } catch (e: Exception) {
                Toast.makeText(context, "未找到应用商店", Toast.LENGTH_SHORT).show()
            }}
        mDialog = builder.setView(view).create()
        initDialog()
    }

    fun showObservationDialog(context: Context, data: ObservationData.ObservationDataBean) {
        UMPostUtils.onEvent(context,"aqi_click")
        mContext = WeakReference(context)
        val builder = AlertDialog.Builder(mContext!!.get(), R.style.DialogStyle)
        val view = LayoutInflater.from(mContext!!.get()).inflate(R.layout.dialog_observation_layout, null, false)
        val pm2 = view.findViewById<TextView>(R.id.pm2_num)
        val pm10 = view.findViewById<TextView>(R.id.pm10_num)
        val o3 = view.findViewById<TextView>(R.id.o3_num)
        val co = view.findViewById<TextView>(R.id.co_num)
        val no = view.findViewById<TextView>(R.id.no_num)
        val no2 = view.findViewById<TextView>(R.id.no2_num)
        val close = view.findViewById<ImageView>(R.id.dialog_close)
        val circleProgress = view.findViewById<CircleProgress>(R.id.circleProgress)
        circleProgress.value = data.Index.toFloat()
        circleProgress.unit = WeatherUtils.getWeatherObservationText(context, data.Index)
        circleProgress.setGradientColors(intArrayOf(WeatherUtils.getWeatherObservationColor(context, data.Index)))
        pm2.text = data.ParticulateMatter2_5.toString()
        pm10.text = data.ParticulateMatter10.toString()
        o3.text = data.Ozone.toString()
        co.text = data.CarbonMonoxide.toString()
        no.text = data.NitrogenDioxide.toString()
        no2.text = data.NitrogenDioxide.toString()
        close.setOnClickListener {
            dialogDismiss()
        }
        mDialog = builder.setView(view).create()
        initDialog()
        circleProgress.startAnimation()
    }

    fun dialogDismiss() {
        if (mDialog != null) {
            if (mDialog!!.isShowing) {
                mDialog!!.dismiss()
                mDialog = null
            }
        }
    }
}