package com.ltt.compass.weather.util


import android.app.Activity
import android.content.Context
import android.content.pm.ActivityInfo
import android.content.res.Configuration
import android.graphics.Color
import android.util.DisplayMetrics
import android.view.WindowManager


object DisplayUtils {
    /**
     * 将px值转换为dp值
     */
    fun px2dp(context: Context, pxValue: Float): Int {
        val scale = context.resources.displayMetrics.density
        return (pxValue / scale + 0.5f).toInt()
    }

    /**
     * 将dp值转换为px值
     */
    fun dp2px(context: Context, dpValue: Float): Int {
        val scale = context.resources.displayMetrics.density
        return (dpValue * scale + 0.5f).toInt()
    }

    /**
     * 将px值转换为sp值
     */
    fun px2sp(context: Context, pxValue: Float): Int {
        val scale = context.resources.displayMetrics.scaledDensity
        return (pxValue / scale + 0.5f).toInt()
    }

    /**
     * 将sp值转换为px值
     */
    fun sp2px(context: Context, dpValue: Float): Int {
        val scale = context.resources.displayMetrics.scaledDensity
        return (dpValue * scale + 0.5f).toInt()
    }


    /**
     * 切换屏幕的方向.
     */
    fun toggleScreenOrientation(activity: Activity) {
        activity.requestedOrientation = if (isPortrait(activity))
            ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE
        else
            ActivityInfo.SCREEN_ORIENTATION_PORTRAIT
    }

    /**
     * 获得当前屏幕的方向.
     *
     * @return 是否竖屏.
     */
    fun isPortrait(context: Context): Boolean {
        val orientation = context.resources.configuration.orientation
        return orientation == Configuration.ORIENTATION_PORTRAIT
    }

    fun getScreenH(context: Context): Int {
        val manager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        val dmh = DisplayMetrics()
        manager.defaultDisplay.getMetrics(dmh)
        return dmh.heightPixels
    }

    fun getScreenW(context: Context): Int {
        val manager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager
        val dmw = DisplayMetrics()
        manager.defaultDisplay.getMetrics(dmw)
        return dmw.widthPixels
    }


    /**
     * 根据滑动变换alpha值
     */
    fun getColorForScrollY(viewHeight: Int, scrollY: Int): Int {
        return when {
            scrollY <= 0 -> {
                Color.argb(0, 0, 0, 0)
            }
            scrollY in 1..viewHeight -> {
                val persent = scrollY * 1f / viewHeight
                val alpha = (100 * persent).toInt()
                Color.argb(alpha, 0, 0, 0)
            }
            else -> {
                Color.argb(100, 0, 0, 0)
            }
        }
    }

    //获取状态栏高度
    fun getStatusBarHeight(context: Context): Int {
        var result = 0
        val resourceId = context.resources.getIdentifier("status_bar_height", "dimen", "android")
        if (resourceId > 0) {
            result = context.resources.getDimensionPixelSize(resourceId)
        }
        return result
    }


}


