package com.ltt.compass.weather.util


import com.dotools.encryption.EncryptionUtil
import com.ltt.compass.weather.AppConfig
import com.ltt.compass.weather.api.ApiConstants

/**
 * 请求表单工具类
 */
object HttpPostRequestUtil {
    private val mEncryptionUtil: EncryptionUtil = EncryptionUtil()//加密工具
    /**
     * 根据经纬度获取城市
     * @param  longitude  经度
     * @param  latitude 纬度
     */
    fun cityForLongitude(longitude: String, latitude: String): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["details"] = "false"
        mPostMap["q"] = "$latitude,$longitude"
        setAppSign(mPostMap)
        return mPostMap
    }

    /**
     * 根据经纬度获取城市
     * @param ip IP地址
     */
    fun cityForIp(ip: String): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["details"] = "false"
        mPostMap["ip"] = ip
        setAppSign(mPostMap)
        return mPostMap
    }

    /**
     * 获取当前天气
     * @param cityId 城市id
     */
    fun getCurrentWeather(cityId: String): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["locationKey"] = cityId
        mPostMap["details"] = "true"
        setAppSign(mPostMap)
        return mPostMap
    }

    /**
     * 获取24小时天气
     * @param cityId 城市id
     */
    fun get24HourWeather(cityId: String, isMetric: Boolean): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["locationKey"] = cityId
        mPostMap["details"] = "true"
        mPostMap["hour"] = "24hour"
        mPostMap["metric"] = isMetric.toString()
        setAppSign(mPostMap)
        return mPostMap
    }


    /**
     * 获取10日天气
     * @param cityId 城市id
     */
    fun get10DayWeather(cityId: String, isMetric: Boolean): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["locationKey"] = cityId
        mPostMap["details"] = "true"
        mPostMap["day"] = "10day"
        mPostMap["metric"] =isMetric.toString()
        setAppSign(mPostMap)
        return mPostMap
    }


    /**
     * 获取指数
     * @param cityId 城市id
     */
    fun indicesDaily(cityId: String, indicesType: WeatherUtils.WeatherIndicesType): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["locationKey"] = cityId
        mPostMap["day"] = "1day"
        when (indicesType) {
            WeatherUtils.WeatherIndicesType.CAPSULE -> {
                mPostMap["groupId"] = "25"
                mPostMap["indexId"] = "25"
            }
            WeatherUtils.WeatherIndicesType.FLIGHT -> {
                mPostMap["groupId"] = "-3"
                mPostMap["indexId"] = "-3"
            }
            WeatherUtils.WeatherIndicesType.FISHING->{
                mPostMap["groupId"] = "13"
                mPostMap["indexId"] = "13"
            }
            WeatherUtils.WeatherIndicesType.TELESCOPE->{
                mPostMap["groupId"] = "12"
                mPostMap["indexId"] = "12"
            }
            WeatherUtils.WeatherIndicesType.SPORTS->{
                mPostMap["groupId"] = "16"
                mPostMap["indexId"] = "16"
            }
            WeatherUtils.WeatherIndicesType.TRIP->{
                mPostMap["groupId"] = "40"
                mPostMap["indexId"] = "40"
            }
        }
        setAppSign(mPostMap)
        return mPostMap
    }

    /**
     * 获取空气质量
     * @param cityId 城市id
     */
    fun observations(cityId: String): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["locationKey"] = cityId
        setAppSign(mPostMap)
        return mPostMap
    }


    /**
     * 获取搜索城市
     * @param cityName 城市
     */
    fun getSearchCity(cityName: String): HashMap<String, String> {
        val mPostMap = HashMap<String, String>()
        mPostMap["location"] = cityName
        mPostMap["countryCodeEnum"] = ""
        setAppSign(mPostMap)
        return mPostMap
    }



    /**
     * 加密
     */
    private fun setAppSign(map: HashMap<String, String>): Boolean {
        return try {
            map["appId"] = ApiConstants.APP_KEY
            map["appSign"] = ApiConstants.APP_SECRET
            map["language"] = AppConfig.WEATHER_LANGUAGE
            map["appSign"] = mEncryptionUtil.makeSign(
                    ApiConstants.APP_KEY, ApiConstants.APP_SECRET,
                    mEncryptionUtil.transformationUri(mEncryptionUtil.sortMapByKey(map))
            )
            true
        } catch (e: Exception) {
            e.printStackTrace()
            false
        }
    }
}