package com.ltt.compass.weather.util


import android.util.Log
import com.ltt.compass.weather.AppConfig
import java.text.ParseException
import java.text.SimpleDateFormat
import java.util.*

object TimeUtil {
    private val MONTH_DAY_FORMAT = SimpleDateFormat("MM-dd")
    private val TIME_FORMAT = SimpleDateFormat("yyyyMMddHHmmss")
    private val DATE_FORMAT = SimpleDateFormat("yyyy-MM-dd HH:mm")
    private val HOUR_MINUTE = SimpleDateFormat("HH:mm")
    private val HOUR = SimpleDateFormat("HH")
    private val WEEK = SimpleDateFormat("EEEE")
    private const val AM_TIP = ""
    private const val PM_TIP = ""
    private const val YESTERDAY = "昨天"
    private const val BEFORE_YESTERDAY = "前天"
    const val DAY_OF_YEAR: Long = 365
    const val DAY_OF_MONTH: Long = 30
    const val HOUR_OF_DAY: Long = 24
    const val MIN_OF_HOUR: Long = 60
    const val SEC_OF_MIN: Long = 60
    const val MILLIS_OF_SEC: Long = 1000
    /**
     * 新时间戳显示
     * 0<X></X><1分钟     ：  刚刚
     * 1分钟<=X<60分钟    :    X分钟前         EX:5分钟前
     * 1小时<=X<24小时    ：  X小时前        EX：3小时前
     * 1天<=X<7天         ：   X天前           Ex： 4天前
     * 1周<=X<1个月     ：   X周前           EX：3周前
     * X>=1个月           :      YY/MM/DD   HH:MM       EX：15/05/15  15:34
     */
    fun halfAnHourSeconds(): Long {
        return MIN_OF_HOUR * SEC_OF_MIN / 2
    }

    fun anHourSeconds(): Long {
        return MIN_OF_HOUR * SEC_OF_MIN
    }

    fun twoHourSeconds(): Long {
        return MIN_OF_HOUR * SEC_OF_MIN * 2
    }

    fun getTimeTips(formatTime: String?): String? {
        var timeTips = formatTime
        try {
            val date = DATE_FORMAT.parse(formatTime)
            timeTips = getTimeTips(date.time)
        } catch (e: ParseException) {
            e.printStackTrace()
        }
        return timeTips
    }

    fun getHourMinute(time: Int): String {
        val date = Date((time.toLong()*1000))
        return HOUR_MINUTE.format(date)
    }

    fun getMonthDay(time: Long): String {
        val date = Date((time*1000))
        return MONTH_DAY_FORMAT.format(date)
    }

    fun getWeek(dataStr: Long): String {
        try {
            val date = Date((dataStr*1000))
            return WEEK.format(date)
        } catch (e: ParseException) {
            e.printStackTrace()
        }
        return ""
    }

    /**
     * 计算事件差判断是否需要刷新
     */
    fun needRefresh(timeStamp: Long): Boolean {
        val time = (System.currentTimeMillis() / 1000) - timeStamp
        if (time >= 600) {//10分钟刷新
            Log.e(AppConfig.WEATHER_TAG, "时间：" + time)
            return true
        }
        return false
    }

    private fun getTimeTips(timeStamp: Long): String {
        val now = System.currentTimeMillis() / 1000
        val tips: String
        var diff = now - timeStamp / 1000
        if (diff < 60) {
            tips = "刚刚"
        } else if (60.let { diff /= it; diff } < 60) {
            tips = String.format("%d分钟前", diff)
        } else if (60.let { diff /= it; diff } < 24) {
            tips = String.format("%d小时前", diff)
        } else if (24.let { diff /= it; diff } < 7) {
            tips = String.format("%d天前", diff)
        } else if (7.let { diff /= it; diff } < 4) {
            tips = String.format("%d周前", diff)
        } else {
            tips = DATE_FORMAT.format(Date(timeStamp * 1000))
        }
        return tips
    }

    fun milliseconds2String(milliSeconds: Long): String {
        return DATE_FORMAT.format(Date(milliSeconds))
    }

    fun isToday(timestamp: Long): Boolean {
        val millOfDay = MILLIS_OF_SEC * SEC_OF_MIN * MIN_OF_HOUR * HOUR_OF_DAY
        return System.currentTimeMillis() / millOfDay == timestamp / millOfDay
    }

    val isNight: Boolean
        get() {
            val currentHour = HOUR.format(Date()).toInt()
            return currentHour >= 19 || currentHour <= 6
        }

    fun isNight(epochSet: Int): Boolean {
        return epochSet > System.currentTimeMillis() / 1000
    }
}