package com.ltt.compass.weather.util


import android.content.Context
import android.graphics.Bitmap
import android.graphics.BitmapFactory
import android.graphics.Color
import androidx.core.content.ContextCompat
import com.ltt.compass.R
import java.util.*

/**
 * 天气相关工具类
 */
object WeatherUtils {

    enum class WeatherViewType {
        DEFAULT, CLEAR_D, CLEAR_N, RAIN_D, RAIN_N, SNOW_D, SNOW_N, CLOUDY_D, CLOUDY_N, OVERCAST_D, OVERCAST_N, FOG_D, FOG_N, HAZE_D, HAZE_N, SAND_D, SAND_N, WIND_D, WIND_N, RAIN_SNOW_D, RAIN_SNOW_N, UNKNOWN_D, UNKNOWN_N
    }


    enum class WeatherIndicesType {
        CAPSULE, FLIGHT, FISHING, TELESCOPE, SPORTS, TRIP
    }

    fun getWeatherIcon(context: Context, code: Int): Int {
        return context.resources.getIdentifier("weather_icon_$code", "mipmap", context.packageName)
    }


    fun getIndicesLevel(indicesType: WeatherUtils.WeatherIndicesType, code: Double): Int {
        when (indicesType) {
            WeatherIndicesType.CAPSULE -> {
                return when (code) {
                    in 0.0..4.0 -> {
                        R.string.prone_level_1
                    }
                    in 4.1..6.0 -> {
                        R.string.prone_level_2
                    }
                    in 6.1..8.0 -> {
                        R.string.prone_level_3
                    }
                    in 8.1..10.0 -> {
                        R.string.prone_level_4
                    }
                    else -> {
                        R.string.unknow
                    }
                }
            }
            WeatherIndicesType.FLIGHT -> {
                return when (code) {
                    in 0.0..3.0 -> {
                        R.string.probability_level_5
                    }
                    in 3.1..5.0 -> {
                        R.string.probability_level_4
                    }
                    in 5.1..7.0 -> {
                        R.string.probability_level_3
                    }
                    in 7.1..9.0 -> {
                        R.string.probability_level_2
                    }
                    in 9.0..10.0 -> {
                        R.string.probability_level_1
                    }
                    else -> {
                        R.string.unknow
                    }
                }
            }
            WeatherIndicesType.FISHING, WeatherIndicesType.SPORTS, WeatherIndicesType.TELESCOPE -> {
                return when (code) {
                    in 0.0..3.0 -> {
                        R.string.suitable_level_1
                    }
                    in 3.1..5.0 -> {
                        R.string.suitable_level_2
                    }
                    in 5.1..7.0 -> {
                        R.string.suitable_level_3
                    }
                    in 7.1..9.0 -> {
                        R.string.suitable_level_4
                    }
                    in 9.0..10.0 -> {
                        R.string.suitable_level_5
                    }
                    else -> {
                        R.string.unknow
                    }
                }
            }
            WeatherIndicesType.TRIP -> {
                return when (code) {
                    in 0.0..3.0 -> {
                        R.string.suitable_level_1
                    }
                    in 3.1..6.0 -> {
                        R.string.suitable_level_2
                    }
                    in 6.1..7.5 -> {
                        R.string.suitable_level_3
                    }
                    in 7.51..9.0 -> {
                        R.string.suitable_level_4
                    }
                    in 9.1..10.0 -> {
                        R.string.suitable_level_5
                    }
                    else -> {
                        R.string.unknow
                    }
                }
            }
        }
    }

    fun getWindLevel(isMetric: Boolean, level: Double): Int {
        var value = level
        if (!isMetric) {
            value *= 1.6
        }
        return when (value.toInt()) {
            in 0..5 -> {
                1
            }
            in 6..11 -> {
                2
            }
            in 12..19 -> {
                3
            }
            in 20..28 -> {
                4
            }
            in 29..38 -> {
                5
            }
            in 39..49 -> {
                6
            }
            in 50..61 -> {
                7
            }
            in 62..74 -> {
                8
            }
            in 75..88 -> {
                9
            }
            in 89..102 -> {
                10
            }
            in 103..117 -> {
                11
            }
            else -> {
                12
            }
        }
    }

    /**
     * 截取风向
     */
    fun getWindOrientation(localized: String): String {
        return if(localized.length>1){
            localized.substring(0,2)+"风"
        }else{
            localized+"风"
        }
    }

    fun getWeatherType(code: Int, isDay: Boolean): WeatherViewType {
        return when (code) {
            1, 2, 3 -> WeatherViewType.CLEAR_D
            33, 34 -> WeatherViewType.CLEAR_N
            12, 13, 14, 15, 16, 17, 18, 26, 39, 40, 41, 42 -> {
                if (isDay) {
                    WeatherViewType.RAIN_D
                } else {
                    WeatherViewType.RAIN_N
                }
            }
            19, 20, 21, 22, 24, 25, 23, 43, 44 -> {
                if (isDay) {
                    WeatherViewType.SNOW_D
                } else {
                    WeatherViewType.SNOW_N
                }
            }
            4, 6, 7, 35, 36, 38 -> {
                if (isDay) {
                    WeatherViewType.CLOUDY_D
                } else {
                    WeatherViewType.CLOUDY_N
                }
            }
            8 -> {
                if (isDay) {
                    WeatherViewType.OVERCAST_D
                } else {
                    WeatherViewType.OVERCAST_N
                }
            }
            11 -> {
                if (isDay) {
                    WeatherViewType.FOG_D
                } else {
                    WeatherViewType.FOG_N
                }
            }
            5, 37 -> {
                if (isDay) {
                    WeatherViewType.HAZE_D
                } else {
                    WeatherViewType.HAZE_N
                }
            }
            32 -> {
                if (isDay) {
                    WeatherViewType.WIND_D
                } else {
                    WeatherViewType.WIND_N
                }
            }
            29 -> {
                if (isDay) {
                    WeatherViewType.RAIN_SNOW_D
                } else {
                    WeatherViewType.RAIN_SNOW_N
                }
            }
            else -> WeatherViewType.UNKNOWN_D
        }
    }

    // 获得0--n之内的不等概率随机整数，0概率最大，1次之，以此递减，n最小
    fun getAnyRandInt(n: Int): Int {
        val max = n + 1
        val bigend = (1 + max) * max / 2
        val rd = Random()
        val x = Math.abs(rd.nextInt() % bigend)
        var sum = 0
        for (i in 0 until max) {
            sum += max - i
            if (sum > x) {
                return i
            }
        }
        return 0
    }

    /**
     * 获取[min, max)内的随机数，越大的数概率越小
     * @param min
     * @param max
     * @return
     */
    fun getDownRandFloat(min: Float, max: Float): Float {
        val bigend = (min + max) * max / 2f
        // Random rd = new Random();
        val x = getRandom(min, bigend) // Math.abs(rd.nextInt() % bigend);
        var sum = 0
        var i = 0
        while (i < max) {
            sum += (max - i).toInt()
            if (sum > x) {
                return i.toFloat()
            }
            i++
        }
        return min
    }

    /**
     * [min, max)
     *
     * @param min
     * @param max
     * @return
     */
    fun getRandom(min: Float, max: Float): Float {
        require(max >= min) {
            "max should bigger than min!!!!"
        }
        return (min + Math.random() * (max - min)).toFloat()
    }

    /**
     * 必须取[0,1]之间的float
     *
     * @param alpha
     * @return
     */
    fun fixAlpha(alpha: Float): Float {
        if (alpha > 1f) {
            return 1f
        }
        return if (alpha < 0f) {
            0f
        } else alpha
    }

    fun convertAlphaColor(percent: Float, originalColor: Int): Int {
        val newAlpha = (percent * 255).toInt() and 0xFF
        return newAlpha shl 24 or (originalColor and 0xFFFFFF)
    }

    /**
     * 获取空气质量描述
     */
    fun getWeatherObservationText(context: Context, observationCode: Int): String {
        when {
            observationCode < 50 -> {
                return context.resources.getString(R.string.observation_good)
            }
            observationCode in 51..100 -> {
                return context.resources.getString(R.string.observation_fine)
            }
            observationCode in 101..150 -> {
                return context.resources.getString(R.string.observation_mild)
            }
            observationCode in 151..200 -> {
                return context.resources.getString(R.string.observation_medium)
            }
            observationCode in 201..300 -> {
                return context.resources.getString(R.string.observation_seriously)
            }
            observationCode in 301..500 -> {
                return context.resources.getString(R.string.observation_serious)
            }
            else -> {
                return context.resources.getString(R.string.unknow)
            }
        }
    }

    /**
     * 获取空气质量颜色
     */
    fun getWeatherObservationBackground(observationCode: Int): Int {
        when {
            observationCode < 50 -> {
                return R.drawable.radius_green_bg
            }
            observationCode in 51..100 -> {
                return R.drawable.radius_yellow_bg
            }
            observationCode in 101..150 -> {
                return R.drawable.radius_orange_bg
            }
            observationCode in 151..200 -> {
                return R.drawable.radius_red_bg
            }
            observationCode in 201..300 -> {
                return R.drawable.radius_purple_bg
            }
            observationCode in 301..500 -> {
                return R.drawable.radius_deep_purple_bg
            }
            else -> {
                return R.drawable.radius_green_bg
            }
        }
    }

    /**
     * 获取空气质量颜色
     */
    fun getWeatherObservationColor(context: Context,observationCode: Int): Int {
        when {
            observationCode < 50 -> {
                return ContextCompat.getColor(context, R.color.great_pollution)
            }
            observationCode in 51..99 -> {
                return ContextCompat.getColor(context, R.color.good_pollution)
            }
            observationCode in 101..150 -> {
                return ContextCompat.getColor(context, R.color.mild_pollution)
            }
            observationCode in 151..200 -> {
                return ContextCompat.getColor(context, R.color.moderate_pollution)
            }
            observationCode in 201..300 -> {
                return ContextCompat.getColor(context, R.color.severe_pollution)
            }
            observationCode in 301..500 -> {
                return ContextCompat.getColor(context, R.color.serious_pollution)
            }
            else -> {
                return Color.WHITE
            }
        }
    }



    /**
     * 根据天气获取对应的图标，并且缩放到指定大小
     * @return
     */
    fun getWeatherIcon(context: Context, weatherIconCode: Int, requestW: Float, requestH: Float): Bitmap {
        val resId = getWeatherIcon(context, weatherIconCode)
        val bmp: Bitmap
        val outWdith: Int
        val outHeight: Int
        val options = BitmapFactory.Options()
        options.inJustDecodeBounds = true
        BitmapFactory.decodeResource(context.resources, resId, options)
        outWdith = options.outWidth
        outHeight = options.outHeight
        options.inSampleSize = 1
        if (outWdith > requestW || outHeight > requestH) {
            val ratioW = Math.round(outWdith / requestW)
            val ratioH = Math.round(outHeight / requestH)
            options.inSampleSize = Math.max(ratioW, ratioH)
        }
        options.inJustDecodeBounds = false
        bmp = BitmapFactory.decodeResource(context.resources, resId, options)
        return bmp
    }

}