package com.ltt.compass.weather.widget


import android.annotation.SuppressLint
import android.content.Context
import android.util.AttributeSet
import android.widget.TextView
import androidx.constraintlayout.widget.ConstraintLayout
import com.ltt.compass.R
import com.ltt.compass.weather.bean.DailyDayWeatherData
import com.ltt.compass.weather.bean.MetricAndImperialBean
import com.ltt.compass.weather.bean.WindBean
import com.ltt.compass.weather.util.SettingConfig
import com.ltt.compass.weather.util.TimeUtil
import com.ltt.compass.weather.util.WeatherUtils

/**
 * 天气详情view 对应weather_details.xml
 */
@SuppressLint("StringFormatInvalid,SetTextI18n")
class WeatherDetailView : ConstraintLayout {
    /**
     * 风级别
     */
    private lateinit var mWeatherDetailWindLevel: TextView
    /**
     * 体感温度
     */
    private lateinit var mWeatherDetailSensationNum: TextView
    /**
     * 相对湿度
     */
    private lateinit var mWeatherDetailHumidityNum: TextView
    /**
     * 日出时间
     */
    private lateinit var mWeatherDetailSunriseTime: TextView
    /**
     * 日落时间
     */
    private lateinit var mWeatherDetailSunsetTime: TextView
    /**
     * 降水概率
     */
    private lateinit var mWeatherDetailRainNum: TextView


    constructor (context: Context?) : super(context)

    constructor (context: Context?, attrs: AttributeSet?) : super(context, attrs)

    constructor (context: Context?, attrs: AttributeSet?, defStyleAttr: Int) : super(context, attrs, defStyleAttr)

    fun initViews() {
        mWeatherDetailWindLevel = findViewById(R.id.weather_detail_wind_level)
        mWeatherDetailSensationNum = findViewById(R.id.weather_detail_sensation_num)
        mWeatherDetailHumidityNum = findViewById(R.id.weather_detail_humidity_num)
        mWeatherDetailSunriseTime = findViewById(R.id.weather_detail_sunrise_time)
        mWeatherDetailSunsetTime = findViewById(R.id.weather_detail_sunset_time)
        mWeatherDetailRainNum = findViewById(R.id.weather_detail_rain_num)
    }

    /**
     * 设置风
     */
    fun setWind(wind: WindBean) {
        val isMetric = SettingConfig.getInstance(context).getTemperatureIsMetric()
        if (isMetric) {
            mWeatherDetailWindLevel.text =  WeatherUtils.getWindOrientation( wind.Direction!!.Localized!!)+" "+resources.getString(R.string.weather_wind_level, WeatherUtils.getWindLevel(isMetric, wind.Speed!!.Metric!!.Value).toString())
        } else {
            mWeatherDetailWindLevel.text = WeatherUtils.getWindOrientation( wind.Direction!!.Localized!!)+" "+ resources.getString(R.string.weather_wind_level, WeatherUtils.getWindLevel(isMetric, wind.Speed!!.Imperial!!.Value).toString())
        }
    }


    /**
     * 设置体感温度
     */
    fun setSensation(sensation: MetricAndImperialBean) {
        if (SettingConfig.getInstance(context).getTemperatureIsMetric()) {
            mWeatherDetailSensationNum.text = sensation.Metric!!.Value.toInt().toString() + resources.getString(R.string.weather_current_degrees_unit, sensation.Metric!!.Unit)
        } else {
            mWeatherDetailSensationNum.text = sensation.Imperial!!.Value.toInt().toString() + resources.getString(R.string.weather_current_degrees_unit, sensation.Imperial!!.Unit)
        }
    }

    /**
     * 设置相对湿度
     */
    fun setHumidity(humidity: String) {
        mWeatherDetailHumidityNum.text = humidity
    }

    /**
     * 设置日出时间 日落时间
     */
    fun setSunTime(sunrise: DailyDayWeatherData.DailyDayWeatherDataBean.DailyForecastsBean.SunBean) {
        mWeatherDetailSunriseTime.text = TimeUtil.getHourMinute(sunrise.EpochRise)
        mWeatherDetailSunsetTime.text = TimeUtil.getHourMinute(sunrise.EpochSet)
    }


    /**
     * 设置下雨概率
     */
    fun setRainNum(rain: String) {
        mWeatherDetailRainNum.text = "$rain%"
    }


}